import sys
sys.path.append('.')
import lvgl as lv
import lv_utils
import time
import uctypes
import struct
import machine

imu_data_update_timer = None
bat_data_update_timer = None

class WIDGETS(object):
    def __init__(self,LCD=None,TSC=None,IMU=None):
        # Init parameter
        self.LCD = LCD
        self.TSC = TSC
        self.IMU = IMU 

        global imu_data_update_timer
        global bat_data_update_timer
        imu_data_update_timer = machine.Timer(-1)
        imu_data_update_timer.init(period=300, mode=machine.Timer.PERIODIC, callback=self.imu_data_update_timer_cb)
        bat_data_update_timer = machine.Timer(-1)
        bat_data_update_timer.init(period=200, mode=machine.Timer.PERIODIC, callback=self.bat_data_update_timer_cb)

        # Style Config
        style_slider = lv.style_t()
        style_slider.init()
        style_slider.set_bg_color(lv.palette_main(lv.PALETTE.ORANGE))
        style_slider.set_border_color(lv.palette_darken(lv.PALETTE.ORANGE, 3))

        style_indic = lv.style_t()
        style_indic.init()
        style_indic.set_bg_color(lv.palette_lighten(lv.PALETTE.DEEP_ORANGE, 3))
        style_indic.set_bg_grad_color(lv.palette_main(lv.PALETTE.DEEP_ORANGE))
        style_indic.set_bg_grad_dir(lv.GRAD_DIR.HOR)

        style_indic_pr = lv.style_t()
        style_indic_pr.init()
        style_indic_pr.set_shadow_color(lv.palette_main(lv.PALETTE.DEEP_ORANGE))
        style_indic_pr.set_shadow_width(10)
        style_indic_pr.set_shadow_spread(3)

        style_sw = lv.style_t()
        style_sw.init()
        style_sw.set_bg_opa(lv.OPA.COVER)
        style_sw.set_bg_color(lv.palette_lighten(lv.PALETTE.RED, 1))
        style_sw.set_shadow_width(55)
        style_sw.set_shadow_color(lv.palette_main(lv.PALETTE.PINK))

        style_roller = lv.style_t()
        style_roller.init()
        style_roller.set_border_color(lv.palette_darken(lv.PALETTE.BLUE, 3))
        style_roller.set_shadow_width(55)
        style_roller.set_shadow_color(lv.palette_main(lv.PALETTE.BLUE_GREY))
        style_roller.set_text_font(lv.font_montserrat_18)

        style_imu_table = lv.style_t()
        style_imu_table.init()
        style_imu_table.set_border_width(1)
        style_imu_table.set_text_font(lv.font_montserrat_16)
        
        style_lable_table = lv.style_t()
        style_lable_table.init()
        style_lable_table.set_text_font(lv.font_montserrat_24)
        
        # Create tileview
        self.scr = lv.obj()
        self.tv = lv.tileview(self.scr)
        self.tv.set_scrollbar_mode(lv.SCROLLBAR_MODE.OFF)

        # Tile1: Just a pic
        self.tile1 = self.tv.add_tile(0, 0, lv.DIR.BOTTOM)

        waveshare_logo_data = (
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFA\x54\xDA\x4C\x99\x3C\x59\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x38\x24\x59\x2C\xB9\x3C\x3A\x5D\x3C\x9E\x9F\xE7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xB6\x7B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5E\xD7\xB9\x44\x18\x1C\x38\x24\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x1C\x8E\x9D\xAE\x7F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xFA\x54\x5B\x65\x5B\x65\x1A\x55\x5E\xD7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x37\x03\x77\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03'
            b'\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x77\x03\x77\x03\x57\x03\xD8\x0B\x7B\x6D\x7F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x37\x03\x77\x03\x97\x03\x3A\x5D\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\x38\x24\x77\x03\x97\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xCF\x97\x03\x57\x03\x37\x03\x79\x34\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\x39\x24\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x2C\x59\x2C\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x16\x03\x77\x03\x77\x03\x16\x03\xDD\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x57\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\x97\x03\x77\x03\x16\x03\x99\x3C\x7D\xAE\x9F\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFA\x4C\x77\x03\xB7\x03\x97\x03\xB7\x03\x3E\xD7\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\x59\x24\x97\x03\xB7\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\x77\x03\xB7\x03\x77\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x37\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x77\x03\x9D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x37\x03\xB7\x03\xB7\x03\x57\x03\xDD\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x7D\xA6\xFC\x8D\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\x1C\x8E\xBB\x7D\x9D\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x57\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\x77\x03\x77\x03\x79\x2C\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xAE\x97\x03\x97\x03\x97\x03\x97\x03\x3C\x96\x9F\xE7\x1C\x8E\x1C\x96\x5E\xD7\x5E\xDF\x59\x24\x97\x03\xB7\x03\xFF\xFF\x3E\xD7\xFC\x8D\xFC\x8D\x7F\xE7\xFF\xFF\x9D\xAE\x57\x03\xB8\x03\x57\x03\x5C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x37\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\x97\x03\x97\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x7D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x37\x03\xB8\x03\xB8\x03\x57\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xDF\x59\x24\x37\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\xD8\x0B\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x57\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\xD8\x0B\xB8\x03\xB7\x03\xB7\x03\x77\x03\x97\x03\xB9\x44\x3E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5E\xD7\x38\x24\x97\x03\xB7\x03\x77\x03\xDA\x4C\xFF\xFF\xDD\xBE\x37\x03\x57\x03\x1C\x8E\x7F\xE7\x59\x24\x97\x03\xB7\x03\xFF\xFF\x1C\x8E\x16\x03\xF6\x02\xBD\xB6\xFF\xFF\xFC\x8D\x57\x03\xB8\x03\x57\x03\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\xB9\x3C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xFA\x4C\xDA\x44\x97\x03\xB7\x03\x97\x03\xD8\x0B\xB9\x44\xFA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x44\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\x37\x03\xB8\x03\xB8\x03\x57\x03\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\x99\x34\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\xB8\x03\x3A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x57\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x57\x03\xFA\x4C\x3A\x65\xB8\x03\xB7\x03\xB8\x03\xB8\x03\x97\x03\x77\x03\xD8\x0B\x5D\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\x99\x34\x97\x03\xB7\x03\x97\x03\xF8\x13\x1E\xC7\xFF\xFF\xDD\xBE\x97\x03\xB8\x03\x3C\x96\x7F\xE7\x59\x24\x97\x03\xB7\x03\xFF\xFF\x3C\x96\x57\x03\x57\x03\xDE\xBE\xFF\xFF\x5B\x65\x77\x03\xB7\x03\x37\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\x97\x03\x97\x03\x97\x03\x59\x2C\x5E\xD7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xDF\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x9F\xE7\x1E\xCF\x57\x03\xB8\x03\xB8\x03\x57\x03\x7D\xA6\x9F\xEF\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xE7\x7F\xDF\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\x79\x34\x77\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x37\x03\x7B\x6D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x57\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\xB9\x3C\xFF\xFF\xDC\x85\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x77\x03\x5A\x65'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x99\x3C\x77\x03\xB7\x03\x97\x03\x97\x03\xFC\x8D\xFF\xFF\xFF\xFF\xDD\xBE\x97\x03\xB8\x03\x3C\x96\x7F\xE7\x59\x24\x97\x03\xB7\x03\xFF\xFF\x3C\x96\x57\x03\x57\x03\xDE\xBE\xFF\xFF\x79\x2C\x97\x03\x97\x03\xF8\x13\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xFE\xC6\xDE\xBE\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xBD\xB6\x97\x03\xB7\x03\x97\x03\x38\x24\x7D\xA6\x1E\xC7\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xFE\xC6\xDD\xBE\x9F\xE7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x16\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x37\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xDF\x79\x34\x97\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x0B\xB7\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x77\x03\xDC\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x77\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\x79\x34\x7F\xDF\xFF\xFF\xBB\x7D\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x57\x03\xFA\x54\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x55\x57\x03\xB7\x03\xB7\x03\x77\x03\x7B\x6D\xFF\xFF\xFF\xFF\xFF\xFF\xDD\xBE\x97\x03\xB8\x03\x3C\x96\x7F\xE7\x59\x24\x97\x03\xB7\x03\xFF\xFF\x3C\x96\x57\x03\x57\x03\xDE\xC6\xFF\xFF\x97\x03\x97\x03\x97\x03\x99\x3C\x7F\xDF\x7F\xE7\x7F\xDF\x7F\xDF\x7F\xDF\x5F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xA6\x97\x03\x37\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\xB7\x03\xB8\x03\xB8\x03\x97\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\xF6\x02\xFC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\xBF\xEF\x9F\xEF\x1C\x8E\xF8\x13\x97\x03\xB8\x03\xB7\x03\x97\x03\xB8\x03\x1C\x8E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x18\x1C\x3E\xD7\xFF\xFF\xFF\xFF\xBB\x7D\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\x3A\x65\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x77\x03\xB7\x03\x77\x03\x3A\x5D\xFF\xFF\x5D\x9E\xFE\xC6\xFF\xFF\xFE\xC6\x97\x03\xB8\x03\x5D\x9E\x9F\xEF\x59\x2C\x97\x03\xB8\x03\xFF\xFF\x5D\x9E\x57\x03\x57\x03\xFE\xC6\x7F\xDF\x37\x03\xB8\x03\xB7\x03\xD8\x0B\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\xB8\x03\x3E\xD7\xFF\xFF\xFF\xFF\x9D\xAE\xD8\x13\x97\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB8\x03\xB8\x03\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\xB8\x03\x97\x03\xB8\x03\xB8\x03\x97\x03\xB8\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\xB8\x0B\x3E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x77\x03\x7B\x6D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\x99\x3C\x16\x03\x1A\x55\xFF\xFF\x3E\xCF\x57\x03\xF8\x13\x3A\x65\x7B\x6D\x97\x03\xB8\x03\x1A\x55\xDC\x85\xF8\x1B\x97\x03\xB8\x03\x3C\x9E\x1A\x55\x77\x03\x57\x03\x1E\xCF\x1C\x96\x77\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x37\x03\x3E\xCF\xFF\xFF\xFF\xFF\x9D\xAE\xF8\x13\x97\x03\xB7\x03\xD8\x0B\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x97\x03\xB7\x03\xB7\x03\xB8\x03\xF8\x13\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\x18\x1C\xF8\x13\x97\x03\xB8\x03\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB8\x03\x97\x03\x77\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x57\x03\x97\x03\xB8\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\xFA\x4C\x57\x03\x97\x03\xB8\x03\xB8\x03\x97\x03\x77\x03\x7D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\xFE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x77\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\xDA\x4C\xBF\xEF\xFF\xFF\x79\x34\x77\x03\x97\x03\x77\x03\x18\x1C\xB7\x03\xB7\x03\xB8\x03\xB8\x0B\xB7\x03\xB7\x03\xB7\x03\xB8\x0B\xB8\x03\xB7\x03\x57\x03\x3E\xD7\x99\x3C\x97\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\x37\x03\x3E\xD7\xFF\xFF\xFF\xFF\x9D\xAE\xD8\x13\x97\x03\x97\x03\x9B\x75\xDF\xFF\xDF\xFF\xDF\xFF\xDF\xFF\xDF\xFF\xDF\xFF\xDF\xFF\xDF\xF7\x3E\xD7\x97\x03\xB7\x03\x97\x03\x59\x2C\xDE\xBE\xDF\xF7\xBF\xF7\xBF\xF7\xBF\xF7\xBF\xF7\xBF\xF7\xDF\xF7\xBF\xEF\x1C\x96\x77\x03\xB8\x03\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\xD8\x0B\x1C\x8E\x3C\x9E\x3C\x9E\x3C\x9E\x3C\x9E\x5C\x9E\xFC\x8D\x77\x03\xB8\x03\xB8\x03\x77\x03\x7B\x6D\x5C\x9E\x3C\x9E\x3C\x9E\x3C\x9E\x3C\x9E\x7D\xA6\xD8\x13\x97\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\xF8\x1B\x77\x03\xB7\x03\xB7\x03\xB7\x03\x77\x03\xD8\x0B\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\xD8\x0B\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\xB7\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\x37\x03\xB7\x03\x97\x03\x97\x03\xFC\x85\xB7\x03\x97\x03\x97\x03\x97\x03'
            b'\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x57\x03\x3E\xD7\x57\x03\xB7\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB7\x03\x97\x03\x37\x03\x3E\xCF\xFF\xFF\xFF\xFF\x9D\xAE\xF8\x13\x97\x03\x97\x03\x1C\x8E\x1E\xCF\x99\x3C\xDA\x4C\xDA\x4C\xDA\x4C\xDA\x4C\xB9\x3C\x7D\xA6\xDF\xF7\x97\x03\x97\x03\x97\x03\x59\x2C\xBF\xEF\x9D\xAE\xFC\x8D\xFC\x8D\xFC\x8D\xFC\x8D\xFC\x8D\xDC\x85\x7F\xE7\xBD\xB6\x57\x03\xB8\x03\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\xF8\x13\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\x37\x03\xB8\x03\xB8\x03\x57\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x18\x1C\x97\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\x18\x1C\x77\x03\x97\x03\xB7\x03\xB7\x03\x97\x03\x57\x03\xFA\x4C\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\xB8\x03\xBB\x7D\xBF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x97\x03\xB7\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\x1A\x55\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x59\x2C\x97\x03\xB7\x03\x77\x03\x7B\x6D\xBD\xB6\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x7B\x6D\x77\x03\xB8\x03\x77\x03\xDC\x7D\xFF\xFF\xFF\xFF\xBB\x7D\x77\x03\x97\x03\x59\x2C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\xD8\x13\x97\x03\x97\x03\x1C\x8E\x7D\xA6\x36\x03\x77\x03\x77\x03\x77\x03\x77\x03\x37\x03\x9B\x75\xBF\xEF\x97\x03\x97\x03\x97\x03\x59\x2C\xBF\xF7\x79\x34\x37\x03\x57\x03\x57\x03\x57\x03\x57\x03\x16\x03\x7D\xA6\xBD\xB6\x57\x03\xB8\x03\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\xF8\x13\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\x37\x03\xB8\x03\xB8\x03\x57\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x18\x1C\x97\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x8D\x57\x03\xB7\x03\xB7\x03\xB7\x03\x77\x03\x18\x1C\x5D\xA6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB7\x03\xB8\x0B\xD8\x13\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x13\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\xB8\x03\xD8\x13\xD8\x0B\xD8\x13\xB8\x0B\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x37\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x97\x03\xB7\x03\x97\x03\xF8\x13\x59\x2C\x59\x2C\x79\x2C\x77\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\xF8\x13\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5D\xA6\x97\x03\x97\x03\x97\x03\xD8\x0B\xFE\xC6\xBF\xEF\x1C\x8E\x1C\x96\x1C\x96\x1C\x96\x1C\x96\x1C\x96\x1C\x96\x1C\x96\x1C\x96\x1C\x96\x3C\x9E\x18\x1C\x97\x03\xB8\x03\x77\x03\xFC\x8D\xFF\xFF\xFF\xFF\xDA\x44\x77\x03\x77\x03\xDA\x44\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\xD8\x13\x97\x03\x97\x03\x1C\x8E\x9D\xAE\x97\x03\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\x97\x03\xDC\x85\xBF\xEF\x97\x03\x97\x03\x97\x03\x59\x2C\xBF\xF7\xDA\x4C\xB8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\x97\x03\x9D\xAE\xBD\xB6\x57\x03\xB8\x03\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\xF8\x13\x9F\xEF\xFF\xFF\xDF\xFF\xDF\xFF\xDF\xFF\xFF\xFF\x7F\xE7\x57\x03\xB8\x03\xB8\x03\x57\x03\xBD\xB6\xFF\xFF\xDF\xFF\xDF\xFF\xDF\xFF\xDF\xFF\xFF\xFF\x18\x1C\x97\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\xB7\x03\x77\x03\x1A\x55'
            b'\x3E\xD7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\x97\x03\x79\x2C\x9D\xAE\xDD\xBE\xBD\xB6\xBD\xB6\xBD\xB6\xFE\xC6\x79\x2C\x97\x03\xB8\x03\xB8\x03\x77\x03\x5B\x65\xDD\xBE\xBD\xB6\xDE\xBE\xBB\x7D\xB8\x0B\x97\x03\xB8\x03\xB8\x03\x97\x03\xF8\x13\x3E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x98\x03\x97\x03\xD8\x0B\xFC\x8D\x7F\xDF\x5E\xD7\x7F\xE7\x39\x2C\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x3A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\x38\x24\x97\x03\xB7\x03\x97\x03\xDA\x44\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\xD8\x0B\x97\x03\xB8\x03\x77\x03\x5B\x65\xFF\xFF\xFF\xFF\x38\x24\x97\x03\x77\x03\x5B\x65\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\xD8\x13\x97\x03\x97\x03\xFC\x85\xFF\xFF\xFE\xC6\x1E\xC7\x1E\xC7\x1E\xC7\x1E\xC7\xFE\xC6\x9F\xE7\xBF\xF7\x97\x03\x97\x03\x97\x03\x59\x2C\x7F\xE7\x5F\xDF\xFE\xC6\x1E\xC7\x1E\xC7\x1E\xC7\x1E\xC7\xFE\xC6\xDF\xF7\x9D\xAE\x57\x03\xB8\x03\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB7\x03\xB8\x0B\xFA\x4C\xFA\x54\xFA\x54\xFA\x54\xFA\x54\x1A\x55\xDA\x4C\x97\x03\xB8\x03\xB8\x03\x97\x03\x99\x3C\x1A\x55\xFA\x54\xFA\x54\xFA\x54\xFA\x54\x1A\x5D\xB8\x0B\xB7\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\x97\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\x97\x03\xB8\x0B\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x77\x03\xB7\x03\x97\x03\x97\x03\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\x79\x2C\x97\x03\xB8\x03\xB8\x03\x77\x03\x5B\x65\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\x1A\x5D\x59\x2C\xB8\x03\x77\x03\x79\x2C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xB6\x37\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x59\x2C\x3E\xD7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xFA\x54\x97\x03\xB7\x03\xB8\x03\x77\x03\xFA\x54\xFF\xFF\xDC\x85\x99\x3C\xB9\x3C\xB9\x3C\xB9\x3C\xB9\x3C\xB9\x3C\xB9\x3C\xB9\x3C\xB9\x3C\xB9\x44\x39\x24\xB7\x03\xB7\x03\xB8\x03\x77\x03\xDA\x4C\xDF\xF7\xFF\xFF\x97\x03\x97\x03\x77\x03\xFC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xA6\x77\x03\x16\x03\x17\x03\xBB\x7D\x5F\xDF\xBB\x75\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xBB\x7D\xFE\xC6\xBF\xF7\x97\x03\x97\x03\x97\x03\x59\x2C\x9F\xEF\x3C\x96\xBB\x7D\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xBB\x75\x3E\xD7\x7D\xA6\xD6\x02\x37\x03\xF6\x02\xDC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\xB7\x03\xB8\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB7\x03\x57\x03\x5D\xA6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x37\x03\xB7\x03\x77\x03\xDA\x44\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5B\x65\x77\x03\xB7\x03\x97\x03\xD8\x0B\x9D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\x77\x03\x97\x03\xB7\x03\x97\x03\xFC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x99\x34\x77\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\x3C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x85\x77\x03\xB7\x03\xB8\x03\xB8\x03\x77\x03\xFA\x54\xFF\xFF\x1A\x55\x77\x03\x77\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\x97\x03\x59\x2C\x9F\xEF\xFF\xFF\x37\x03\xB7\x03\x57\x03\x7D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x37\x03\x97\x03\x97\x03\x97\x03\x97\x03\x57\x03\xDC\x7D\xBF\xEF\x97\x03\x97\x03\x97\x03\x59\x2C\xBF\xEF\x38\x24\x77\x03\x97\x03\x97\x03\x97\x03\x97\x03\x37\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB8\x03\x97\x03\x79\x34\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x99\x3C\x77\x03\x77\x03\x3C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDE\xBE\xF8\x13\x97\x03\x77\x03\xDA\x44\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x1A\x55\x77\x03\xB7\x03\x97\x03\xF8\x13\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\x77\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\xFA\x4C\xBF\xF7\xFF\xFF\xFF\xFF\xBD\xB6\x77\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\xFA\x54\xFF\xFF\x1A\x5D\x77\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x7F\xDF\x5E\xD7\x37\x03\x97\x03\x97\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x96\x97\x03\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\x97\x03\xFC\x8D\xBF\xEF\x97\x03\x97\x03\x77\x03\x59\x2C\xBF\xEF\x79\x34\xB8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\x97\x03\x5D\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB7\x03\xB7\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\xF8\x13\x97\x03\xB8\x03\xB8\x03\x97\x03\x59\x2C\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x39\x24\x18\x1C\xDC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB8\x03\xB7\x03\x57\x03\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xAE\x37\x03\x18\x1C\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x79\x34\x97\x03\xB8\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\xB8\x03\x97\x03\xB8\x03\x57\x03\xBB\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xDA\x44\x77\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x18\x24\x5E\xDF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\xFA\x54\xFF\xFF\x3B\x65\xB8\x03\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xD8\x0B\xB8\x03\x9B\x75\x57\x03\xF8\x1B\x97\x03\x57\x03\xDE\xBE\x3C\x96\x57\x03\x97\x03\x18\x1C\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xE7\xBD\xB6\xDD\xBE\xDD\xBE\xDD\xBE\xDD\xBE\xBD\xB6\x7F\xE7\xDF\xF7\xD8\x0B\xD8\x0B\xB8\x03\x79\x34\x9F\xE7\x1E\xCF\xDD\xBE\xDD\xBE\xDD\xBE\xDD\xBE\xDD\xBE\xBD\xB6\x9F\xE7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\xD8\x0B\xDC\x85\xFC\x8D\xFC\x8D\xFC\x8D\xFC\x8D\x1C\x8E\xDB\x7D\x77\x03\xB8\x03\xB8\x03\x77\x03\x5B\x65\x1C\x8E\xFC\x8D\xFC\x8D\xFC\x8D\xFC\x8D\x3C\x9E\xD8\x0B\x97\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x85\x77\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x77\x03\x7B\x6D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03'
            b'\xB8\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x77\x03\xFA\x4C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDC\x85\x77\x03\x18\x1C\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x57\x03\xB7\x03\xB7\x03\x57\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5D\x9E\xB8\x03\x97\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\xDE\xBE\xFF\xFF\xDF\xF7\x38\x24\x97\x03\xB8\x03\x97\x03\xB7\x03\xB8\x03\x77\x03\xFA\x54\xFF\xFF\x1E\xCF\xBD\xB6\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDD\xBE\x9D\xAE\xFF\xFF\xFA\x4C\xBB\x7D\x77\x03\x77\x03\xFC\x85\xFA\x54\x77\x03\x97\x03\x99\x34\x7F\xE7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xBE\xDD\xBE\xBD\xBE\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\x18\x1C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x37\x03\xB8\x03\xB8\x03\x57\x03\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x18\x1C\x97\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x8D\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\x7B\x6D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x97\x03\xB8\x03\xB8\x03\x97\x03\x59\x2C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x55\xDC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xF7\x57\x03\x7B\x6D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x39\x24\x79\x34\x97\x03\x77\x03\x1A\x55\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xF7\x99\x3C\x77\x03\xB8\x03\xB8\x03\xB8\x03\x57\x03\xFC\x8D\xFF\xFF\xFF\xFF\xBD\xB6\x77\x03\x97\x03\x18\x1C\xB8\x03\xB7\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\xDF\xF7\xBD\xB6\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xDE\xBE\xFF\xFF\xFF\xFF\xDF\xFF\xBD\xB6\x57\x03\x97\x03\xDA\x4C\xD8\x13\x97\x03\x77\x03\x5B\x65\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\x9D\xAE\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xDE\xBE\xDE\xBE\xDE\xBE\xDD\xBE\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\xBD\xB6\x9D\xAE\x5E\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\xF8\x13\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\x37\x03\xB8\x03\xB8\x03\x57\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x18\x1C\x97\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x85\x77\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x57\x03\x7B\x6D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xB7\x03\x97\x03\xB8\x03\xB8\x03\x97\x03\x97\x03\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5E\xD7\x1E\xC7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x59\x2C\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\x9B\x75\xB9\x3C\x97\x03\x37\x03\x7F\xE7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\xB7\x03\x97\x03\xB8\x03\xB8\x03\x77\x03\x3A\x5D\xFF\xFF\xFF\xFF\xDF\xF7\xFA\x4C\x77\x03\xBD\xB6\xD8\x13\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\xDD\xBE\x57\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x0B\x1E\xCF\xFF\xFF\xFF\xFF\x3E\xCF\x97\x03\x97\x03\xB8\x03\x97\x03\xB7\x03\x97\x03\x3C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5D\xA6\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03'
            b'\xB8\x03\x97\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\x18\x1C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x37\x03\xB8\x03\xB8\x03\x57\x03\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x18\x1C\x97\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x18\x1C\x97\x03\xB8\x03\xB7\x03\x97\x03\x79\x34\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x59\x2C\x39\x24\xFC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xB7\x03\x97\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\xDC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x59\x2C\x7D\xA6\x9B\x75\x77\x03\x3A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x59\x2C\x97\x03\xB7\x03\xB8\x03\x77\x03\xDA\x44\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x9D\xAE\xFF\xFF\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\xDD\xBE\x37\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\x1E\xCF\xFF\xFF\xFF\xFF\x7F\xE7\x79\x2C\x97\x03\xB7\x03\xB7\x03\x97\x03\x38\x24\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5D\x9E\x57\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB7\x03\xB7\x03\x39\x24\x59\x24\x39\x24\x39\x24\x39\x24\x59\x24\x39\x24\x97\x03\xB8\x03\xB8\x03\x97\x03\x18\x1C\x59\x24\x39\x24\x39\x24\x39\x24\x39\x24\x59\x2C\xB7\x03\xB7\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5D\xA6\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xB7\x03\x97\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x79\x34\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\xBB\x7D\x7B\x6D\xBB\x7D\xBA\x44\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xFC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xAE\x77\x03\xB7\x03\xB8\x03\x97\x03\x99\x34\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xFF\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\xDD\xBE\x57\x03\xB7\x03\xB7\x03\x18\x1C\x39\x24\x39\x24\x97\x03\x97\x03\xB8\x03\x3E\xCF\xFF\xFF\xFF\xFF\xDF\xF7\x3A\x5D\x77\x03\xB8\x03\xB7\x03\x77\x03\xBA\x44\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\x16\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x97\x03\xB8\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xB8\x0B\x97\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\xF8\x13\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xB6\xFA\x4C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\x5B\x65\x3C\x96\x5D\x9E\xFC\x8D\x9D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\x77\x03\x3E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x18\x1C\x97\x03\xB8\x03\x97\x03\x39\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\xDD\xBE\x57\x03\x97\x03\xB8\x0B\x5D\x9E\x5E\xD7\x1E\xCF\x77\x03\x97\x03\xB8\x03\x3E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x8D\x77\x03\xB7\x03\xB7\x03\x97\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\xBB\x7D\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\xDC\x85\x1C\x8E\x59\x2C\x97\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x57\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xF8\x13\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\xDC\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBA\x44\xD8\x13\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5F\xDF\x1C\x96\xBB\x7D\x7B\x6D\x7D\xA6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFA\x4C\x57\x03\x99\x34\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xB6\x37\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\xDD\xBE\x57\x03\x97\x03\xD8\x0B\x1E\xCF\xFF\xFF\xFF\xFF\x77\x03\x97\x03\xB8\x03\x7F\xDF\xDF\xF7\x3C\x9E\xFF\xFF\xBD\xB6\xD8\x0B\x97\x03\x97\x03\xD8\x0B\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3A\x5D\x97\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\xB8\x03\x97\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x37\x03\x5D\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x79\x34\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\xDA\x4C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xFF\xB8\x0B\x97\x03\x7D\xA6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\xBB\x7D\xBD\xB6\x3C\x96\x9D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDE\xBE\xF8\x13\x97\x03\x77\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x59\x2C\x97\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\xDD\xB6\x57\x03\x97\x03\xD8\x13\x1E\xCF\xFF\xFF\xFF\xFF\x77\x03\x97\x03\xD8\x0B\x7B\x6D\x18\x1C\x39\x24\xFF\xFF\x5E\xDF\x79\x2C\x97\x03\x97\x03\x18\x1C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xC7\xFA\x54\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x3A\x5D\x5B\x65\x18\x1C\x97\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x57\x03\xB8\x03\x97\x03\xD8\x0B\x3B\x65\x7B\x6D\x5B\x6D\x5B\x6D\x5B\x6D\x7B\x6D\x3B\x65\x77\x03'
            b'\xB8\x03\xB8\x03\x97\x03\xFA\x4C\x7B\x6D\x5B\x6D\x5B\x6D\x5B\x6D\x5B\x6D\x5B\x6D\x5B\x6D\x5B\x6D\x7B\x6D\x99\x34\xFA\x54\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x55\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x18\x1C\xFE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x8D\x97\x03\x77\x03\xBA\x44\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x79\x2C\xB9\x3C\x7B\x6D\x3C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5B\x6D\x77\x03\xB7\x03\x77\x03\x99\x3C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDA\x44\x97\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\x3C\x9E\x57\x03\x97\x03\x18\x1C\x3E\xD7\xFF\xFF\xFF\xFF\x37\x03\xB7\x03\xB7\x03\x77\x03\x77\x03\xB8\x0B\x9F\xE7\xDC\x85\x77\x03\xB7\x03\x97\x03\xD8\x0B\x7D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5D\xA6\x36\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x97\x03\xB7\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x37\x03\x97\x03\x77\x03\xF8\x13\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x37\x03\xB8\x03\xB8\x03\x57\x03\x1E\xC7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x96\x77\x03\x16\x03\x3A\x5D\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9B\x75\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\xFC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x99\x3C\x97\x03\x97\x03\x18\x1C\x1E\xC7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\xD8\x0B\x97\x03\x5C\x9E\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\xF8\x13\x97\x03\xB7\x03\x77\x03\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\x37\x03\xB7\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\x5B\x6D\x77\x03\x97\x03\x79\x34\x7F\xDF\xFF\xFF\x9D\xAE\x57\x03\xB8\x03\xB7\x03\xB7\x03\x77\x03\x79\x2C\xFC\x8D\x18\x1C\x97\x03\xB8\x03\xB8\x03\x77\x03\x79\x34\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5D\xA6\x37\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB8\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\x38\x24\x79\x34\x79\x2C\xBA\x44\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\x37\x03\xB8\x03\xB8\x03\x57\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9B\x75\x77\x03\xB7\x03\x57\x03\x59\x2C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5C\x9E\x57\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\xDA\x4C\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\x18\x1C\x97\x03\xB7\x03\x97\x03\xFC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5B\x65\x77\x03\x77\x03\x3B\x65\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x8D\x57\x03\xB7\x03\x97\x03\xF8\x13\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x79\x34\x97\x03\xB8\x03\x97\x03\x38\x24'
            b'\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xF7\xFF\xFF\x59\x2C\x97\x03\x77\x03\xDA\x4C\xBF\xEF\xFF\xFF\xB8\x0B\x97\x03\xB8\x03\x97\x03\x77\x03\xFA\x4C\x5F\xDF\x1A\x55\x77\x03\xB8\x03\xB7\x03\xB8\x03\x97\x03\xB7\x03\xFC\x8D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x39\x24\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x79\x34\x99\x3C\xD8\x0B\x97\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xBF\xF7\xBF\xF7\xBF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x37\x03\xB8\x03\xB8\x03\x57\x03\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3A\x5D\x77\x03\xB7\x03\x97\x03\xDA\x4C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5E\xDF\x57\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\xB8\x0B\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDC\x85\x97\x03\xB7\x03\xB8\x03\x77\x03\xDA\x4C\xBF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\x18\x1C\x97\x03\x97\x03\x18\x1C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xF8\x13\x97\x03\xB8\x03\x57\x03\x1C\x8E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\x77\x03\xB7\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xDF\xFF\xBF\xF7\x97\x03\x97\x03\x77\x03\x7B\x6D\xFF\xFF\xFF\xFF\x18\x1C\x97\x03\x97\x03\xB7\x03\xBB\x7D\xFF\xFF\xBB\x7D\x77\x03\xB7\x03\xB7\x03\x77\x03\x97\x03\xB8\x03\x97\x03\x18\x1C\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\x9F\xE7\xDF\xFF\xDA\x44\x97\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x37\x03\xB8\x03\xB8\x03\x57\x03\x5D\xA6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xDF\x39\x24\x97\x03\xB7\x03\x97\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\x57\x03\xB8\x03\x97\x03\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x59\x2C\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xDA\x4C\x77\x03\xB7\x03\xB8\x03\x97\x03\xD8\x0B\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x96\x77\x03\xB7\x03\x97\x03\xB8\x03\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\x57\x03\x97\x03\x97\x03\x77\x03\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x99\x3C\x97\x03\xB7\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xFF\xFF\x9B\x75\x97\x03\x97\x03\x97\x03\x1C\x96\xFF\xFF\xFF\xFF\x3D\x9E\x16\x03\xF8\x1B\xFE\xC6\xFF\xFF\x7B\x75\x77\x03\x97\x03\xB7\x03\x57\x03\x5D\xA6\xF8\x13\x97\x03\xB7\x03\x97\x03\xD8\x0B\x1E\xC7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x7F\xE7\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\x9F\xEF\xDF\xFF\xDA\x4C\x97\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x16\x03\xB8\x03\xB8\x03\x97\x03\x18\x1C\x5D\xA6\x7D\xA6\x7D\xA6\x7D\xA6\x7D\xA6\x7D\xA6\x7D\xA6\x5C\x9E\xBA\x44\x97\x03\xB7\x03\x97\x03\xB8\x03\x5D\xA6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\x7F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x5D\x77\x03\xB8\x03\x97\x03\xF8\x13\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9B\x75\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x79\x34\xDF\xF7\xFF\xFF\xFF\xFF\xDD\xBE\xF8\x13\x97\x03\xB8\x03\xB8\x03\xB7\x03\x57\x03\xDC\x85\xFF\xFF\xFF\xFF\x9F\xEF\x79\x34\x97\x03\xB8\x03\xB7\x03\x97\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xF8\x13\x77\x03\x79\x34\x38\x24\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xA6\xB8\x0B\x97\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\x1A\x5D\xBD\xB6\xF8\x13\x97\x03\x97\x03\x38\x24\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\x39\x24\xDF\xF7\xFF\xFF\x1A\x55\x77\x03\x97\x03\xB7\x03\x77\x03\x7B\x6D\xFF\xFF\x3E\xD7\x37\x03\xB7\x03\xB7\x03\x97\x03\x16\x03\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xB9\x3C\x37\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x97\x03\xB7\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xF8\x13\x97\x03\xB8\x03\xB7\x03\x97\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x97\x03\xB7\x03\xB8\x03\x97\x03\xD8\x0B\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xE7\xF6\x02\x57\x03\x57\x03\x57\x03\x57\x03\x77\x03\x97\x03\xB7\x03\xB8\x03\x97\x03\x59\x24\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x77\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x18\x24\x38\x24\x39\x24\xF8\x13\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\xD8\x0B\x18\x24\x39\x24\x18\x1C\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x77\x03\x37\x03\x37\x03\x37\x03\x37\x03\x97\x03\x77\x03\xDC\x85\x9F\xE7\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xB9\x44\x77\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\x3A\x5D\x1C\x8E\xB8\x0B\x97\x03\x97\x03\x1A\x55\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xFF\xFF\x7B\x6D\x57\x03\x97\x03\xB7\x03\x77\x03\x99\x3C\x9F\xE7\xFF\xFF\xFF\xFF\x1A\x5D\x77\x03\xB7\x03\x97\x03\x97\x03\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDA\x44\x77\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x57\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\x57\x03\x1A\x55\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFA\x54\x77\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\x57\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xE7\x99\x3C\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5D\xA6\xD8\x0B\x97\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xD8\x0B\x97\x03\x77\x03\xFA\x54\xFF\xFF\x1C\x96\x57\x03\xB8\x0B\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\x18\x1C\x97\x03'
            b'\x97\x03\x38\x24\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xFF\xFA\x4C\x77\x03\x77\x03\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xB9\x3C\x37\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x97\x03\xB8\x03\xB7\x03\x77\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x18\x1C\x57\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x57\x03\x18\x1C\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\x77\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\x97\x03\x77\x03\xB7\x03\x5F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x8E\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xA6\x18\x1C\xB7\x03\xB7\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x97\x03\x77\x03\x57\x03\xDA\x4C\xDF\xF7\xFF\xFF\xDA\x4C\x1A\x55\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x57\x03\x39\x24\x3E\xD7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xB9\x3C\xFA\x54\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5C\x9E\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x9B\x75\xDC\x7D\x38\x24\x77\x03\x97\x03\x57\x03\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\x9B\x6D\x39\x24\x57\x03\x37\x03\x37\x03\x37\x03\x37\x03\x37\x03\x37\x03\x37\x03\x37\x03\x37\x03\xB8\x03\x3A\x5D\x3E\xD7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x18\x1C\x57\x03\x77\x03\x77\x03\x57\x03\x57\x03\xD8\x0B\xBA\x44\x9D\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xE7\x59\x2C\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5F\xDF\x59\x2C\x97\x03\xB7\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5B\x65\x3A\x5D\x1A\x5D\x1C\x96\xDF\xFF\xFF\xFF\xDF\xF7\x9F\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5B\x65\xBF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5B\x65\x18\x1C\x39\x24\x18\x1C\xFC\x8D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xEF\x5E\xDF\x5E\xD7\x5E\xDF\x5E\xDF\x5E\xDF\x5E\xDF\x5E\xDF\x5E\xDF\x5E\xDF\x5E\xD7\x7F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1C\x96\x1A\x5D\x7B\x6D\xBB\x7D\x3C\x96\xFE\xC6\x7F\xE7\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x85\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xCF\x18\x24\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFA\x4C\x77\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\x97\x03\x77\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x18\x1C\x77\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x9B\x75\xFF\xFF\xFF\xFF\x5D\x9E\x77\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\xBB\x7D\xFF\xFF\xBB\x7D\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xAE\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x77\x03\x3A\x5D\x7B\x6D\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xDF\xF7\xFE\xC6\xDF\xF7\xFF\xFF\xFF\xFF\xDE\xBE\x9F\xE7\xFF\xFF\xFF\xFF\xBD\xB6\xDF\xF7\xFF\xFF\x5F\xDF\x1E\xCF\xFF\xFF\xFF\xFF\x9D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\x3E\xD7\x3E\xCF\x5C\x9E\x5C\x9E\x5C\x9E\x5D\x9E\x3E\xD7\xFF\xFF\xFF\xFF\xBD\xB6\x7B\x6D\x3E\xCF\xFF\xFF\xFF\xFF\x3E\xD7\x5E\xD7\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\xBF\xEF\xFF\xFF\xFF\xFF\x9F\xEF\xDE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\x5D\x9E\x5C\x9E\x5D\x9E\xFE\xC6\xFF\xFF\xFF\xFF\x7F\xE7\x5C\x9E\x5C\x9E\x5C\x9E\x3C\x9E\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xA6\x5D\x9E\x5C\x9E\x5C\x9E\x1C\x96\xFF\xFF\x9F\xE7\x1E\xC7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDE\xBE\x5C\x9E\x5C\x9E\x5C\x9E\x3C\x96\x7F\xE7\xFF\xFF\xFF\xFF\x9F\xE7\xBB\x7D\xFC\x8D\xFF\xFF\xFF\xFF\x1E\xCF\x3C\x96\x5D\xA6\x7D\xAE\x3C\x9E\x7D\xA6\x9F\xE7\xDE\xBE\x5C\x9E\x5C\x9E\x7D\xA6\x7F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x8D\x7B\x75\xDE\xBE\xFF\xFF\xFF\xFF\xDF\xFF\xFE\xC6\xBF\xF7\xFF\xFF\xFF\xFF\xBF\xF7\xDE\xBE\xFF\xFF\x1E\xCF\x5F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFC\x8D\xBB\x7D\x9F\xE7\xFF\xFF\xFF\xFF\xFF\xFF\x5E\xDF\xBB\x7D\x7D\xA6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\xB8\x0B\x97\x03'
            b'\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\x97\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xDF\xF7\xFA\x4C\x1E\xCF\xFF\xFF\x9F\xE7\x57\x03\x1C\x96\xFF\xFF\xFF\xFF\x77\x03\xFF\xFF\xFF\xFF\x9B\x75\xD8\x0B\xBF\xF7\xFF\xFF\xDA\x44\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\x59\x24\xFE\xC6\x5B\x6D\xDA\x4C\xBB\x7D\xBB\x7D\xBB\x7D\xFE\xC6\xFF\xFF\x3A\x5D\xBA\x44\x9B\x75\xD8\x13\xBB\x7D\xFF\xFF\xBB\x7D\xDC\x7D\xFF\xFF\xFF\xFF\xDF\xF7\xFA\x4C\xFE\xC6\xFF\xFF\xFF\xFF\xBB\x7D\x18\x24\x3E\xD7\xFF\xFF\xFF\xFF\x9B\x75\x99\x3C\xDC\x85\x9B\x75\xF8\x13\xDA\x44\xFF\xFF\x3C\x9E\x59\x2C\xBB\x7D\xBB\x7D\x9B\x75\x3C\x9E\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x9B\x75\xBB\x7D\xBB\x7D\x7B\x6D\xFF\xFF\x7D\xA6\x1A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDA\x4C\xFA\x54\xBB\x7D\xBB\x7D\x9B\x75\x3E\xD7\xFF\xFF\x9D\xAE\xF8\x13\x1A\x5D\x99\x34\xD8\x0B\x9F\xEF\xFE\xC6\x9B\x75\x5B\x6D\xD8\x0B\xDC\x85\xFC\x8D\x3E\xD7\x79\x34\x7B\x6D\xBB\x7D\x5B\x65\xB8\x03\x3C\x9E\xFF\xFF\xFF\xFF\x59\x2C\x39\x24\x5B\x6D\x59\x2C\x3A\x5D\xFF\xFF\x9F\xEF\x18\x1C\xBB\x7D\xFF\xFF\xFF\xFF\x1E\xCF\x79\x34\xFF\xFF\x1A\x55\x7D\xA6\xFF\xFF\x7F\xDF\xF8\x13\xBA\x44\xDA\x4C\x57\x03\x3E\xD7\xFF\xFF\x5D\x9E\x79\x34\x7B\x6D\x39\x24\x79\x34\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\x38\x24\x77\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\x3A\x65\x7D\xA6\xFF\xFF\x7D\xA6\x79\x34\x9B\x75\xFF\xFF\x1E\xCF\x99\x34\xFF\xFF\x3E\xD7\xBB\x7D\x59\x34\xBD\xB6\xFF\xFF\x3C\x96\x3B\x65\xFF\xFF\xFF\xFF\xBF\xEF\xD8\x13\xFF\xFF\x7B\x6D\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDE\xBE\x3A\x5D\xFF\xFF\xFF\xFF\xDF\xF7\x3A\x5D\xBF\xEF\xBB\x7D\xDC\x85\xFF\xFF\xFF\xFF\xFF\xFF\xFA\x4C\xFE\xCE\xFF\xFF\xBF\xEF\xFA\x54\x1A\x55\x3C\x96\xFF\xFF\xFF\xFF\x9B\x75\x3C\x96\xFF\xFF\xFF\xFF\xFF\xFF\x57\x03\xFF\xFF\x3C\x96\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xA6\x1A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x5D\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xCF\xBA\x44\x7F\xDF\xFF\xFF\xFF\xFF\xDC\x85\x7B\x6D\xFF\xFF\xFF\xFF\x9F\xEF\x18\x1C\xFF\xFF\xFF\xFF\xFF\xFF\x79\x2C\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x1A\x5D\xFF\xFF\x1C\x8E\x1A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\x99\x3C\x9D\xAE\xDF\xF7\x79\x34\x18\x1C\x1E\xC7\xFF\xFF\x1E\xCF\x79\x34\xFF\xFF\x1A\x55\x7D\xAE\xFF\xFF\xF8\x1B\xFE\xBE\xFF\xFF\xFF\xFF\x7D\xA6\xBA\x44\xFF\xFF\x3B\x65\xFE\xC6\xFF\xFF\xFF\xFF\xFA\x4C\xDE\xBE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFE\xC6\xBA\x3C\x77\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\xDC\x85\xFF\xFF\x3A\x5D\x5C\x9E\x9B\x7D\x9F\xEF\xFC\x8D\xDC\x85\xFF\xFF\x7D\xA6\xBB\x7D\xBD\xB6\xBA\x44\xFF\xFF\x5E\xD7\xDA\x4C\xBF\xEF\xFF\xFF\x7B\x6D\x3C\x96\xFF\xFF\x5B\x65\x1C\x8E\x7F\xDF\x3E\xD7\x5E\xDF\xFF\xFF\x9F\xE7\xDA\x4C\x1C\x96\xFE\xC6\xBF\xEF\xFF\xFF\xFF\xFF\xDC\x85\x3B\x5D\xFE\xC6\xDE\xBE\xBD\xB6\x99\x3C\x1E\xCF\xFF\xFF\xDE\xBE\xFA\x4C\xFE\xBE\x5B\x65\xDF\xF7\xFF\xFF\x9B\x75\xDC\x85\xFF\xFF\xFF\xFF\x5E\xD7\xB8\x0B\xFF\xFF\x3C\x9E\x1A\x5D\x5F\xDF\x5E\xD7\x5E\xD7\x9F\xEF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xD8\x0B\x1E\xCF'
            b'\x5E\xDF\x3E\xD7\x5E\xDF\xFF\xFF\x7D\xA6\x1A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x55\x3C\x9E\x7F\xDF\x5E\xD7\x3E\xD7\xFF\xFF\x7D\xA6\x1A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xDF\xF8\x1B\xFF\xFF\xFF\xFF\xDF\xF7\x79\x2C\x7F\xE7\xFF\xFF\xDF\xFF\xBB\x7D\x7B\x6D\xFF\xFF\xD8\x0B\xBF\xEF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xB6\xB9\x3C\xDF\xF7\x5B\x65\x7B\x6D\x3A\x5D\xFF\xFF\x3E\xD7\x79\x34\xFF\xFF\x1A\x55\x9D\xAE\xDF\xF7\xB7\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x5B\x65\xDE\xBE\x5F\xDF\xFF\xFF\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xDC\x85\xD8\x13\x77\x03\x97\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xD7\x3A\x5D\xFF\xFF\x18\x1C\xFF\xFF\xBB\x7D\x1E\xCF\xDA\x44\x5E\xD7\xFF\xFF\xBB\x7D\xBD\xB6\xFF\xFF\xF8\x1B\xBF\xF7\xDF\xF7\x7B\x6D\xBD\xB6\xFF\xFF\xD8\x0B\xFF\xFF\xFF\xFF\x3B\x65\xB9\x44\x9B\x75\x7B\x6D\xBB\x75\x7F\xDF\xFF\xFF\xFE\xC6\x7B\x6D\xB9\x3C\x99\x3C\xDE\xBE\xFF\xFF\xDC\x85\x39\x24\xFA\x54\xFA\x4C\xDA\x4C\xF8\x13\x3E\xCF\xFF\xFF\x5B\x65\xBD\xB6\xFF\xFF\xFA\x4C\x3E\xD7\xFF\xFF\x9B\x75\x79\x34\x7B\x6D\xDA\x4C\x5B\x65\xBD\xB6\xFF\xFF\x5C\x9E\x59\x24\x9B\x75\x7B\x6D\x7B\x6D\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x97\x03\x5B\x6D\x9B\x75\x7B\x6D\xBB\x7D\xFF\xFF\x7D\xA6\x1A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDA\x4C\xFA\x4C\x9B\x75\x7B\x6D\x5B\x65\xFF\xFF\x1C\x96\x5B\x65\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xDF\xF8\x1B\xFF\xFF\xFF\xFF\xFF\xFF\x79\x2C\x3A\x5D\x5B\x65\xDA\x4C\x9B\x75\x5E\xD7\xFF\xFF\x57\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5F\xDF\xF8\x1B\xFF\xFF\x3A\x5D\x5E\xD7\xDA\x44\x7D\xAE\x5F\xDF\x79\x34\xFF\xFF\x1A\x55\x9D\xAE\x5E\xD7\x18\x1C\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xDB\x7D\xFA\x4C\x59\x2C\xFC\x8D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\x1A\x55\x77\x03\x77\x03\x97\x03\xB7\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x99\x3C\x3E\xD7\x99\x3C\xFF\xFF\xDC\x85\x5D\x9E\x99\x3C\xFF\xFF\xBF\xEF\xD8\x0B\xF8\x13\x79\x34\x37\x03\xFC\x8D\xFF\xFF\x1C\x96\xFC\x8D\x1E\xCF\xBA\x44\xFF\xFF\xDF\xF7\x5B\x65\x3C\x9E\xBF\xEF\x7F\xE7\x9F\xE7\xFF\xFF\xDF\xF7\xFF\xFF\xFF\xFF\x7F\xE7\x5D\xA6\xFA\x54\x7F\xE7\xDC\x85\x7B\x6D\xBF\xF7\x9F\xEF\x7F\xDF\xDA\x44\x1E\xCF\xFF\xFF\x57\x03\x18\x1C\x59\x2C\x57\x03\xDC\x85\xFF\xFF\xBB\x75\x7B\x6D\x7F\xDF\xDC\x85\x39\x24\xFF\xFF\xFF\xFF\x3C\x96\x3A\x5D\x9F\xEF\x9F\xE7\x7F\xE7\xBF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xD8\x0B\x5E\xD7\x9F\xE7\x7F\xE7\x9F\xE7\xFF\xFF\x7D\xA6\x1A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x55\x7D\xA6\xBF\xEF\x9F\xE7\x7F\xE7\xFF\xFF\x7D\xA6\x1A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7F\xE7\xFF\xFF\xFF\xFF\x7F\xDF\xF8\x1B\xFF\xFF\xFF\xFF\xDF\xF7\x59\x2C\xDE\xBE\x3E\xD7\x5B\x65\x5B\x65\xFF\xFF\xFF\xFF\x77\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3E\xCF\x59\x2C\xDF\xFF\x3A\x5D\x9F\xE7\x5E\xD7\xF8\x13\x5E\xD7\x99\x34\xFF\xFF\x1A\x55\x9D\xAE\xDF\xF7\xB8\x03\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x5E\xD7\xFF\xFF\xBF\xF7\xFF\xFF\xBF\xEF\x3E\xCF\xBA\x44\x7D\xA6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9F\xE7\x9B\x75\x77\x03\x77\x03\x97\x03\x97\x03\x97\x03\x97\x03\xB7\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03'
            b'\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\xB8\x03\x97\x03\x38\x24\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3A\x5D\x3A\x5D\x3C\x96\xFF\xFF\xBD\xB6\x3A\x5D\x3A\x5D\xFF\xFF\x9D\xAE\xBA\x44\xFF\xFF\xDF\xF7\x3E\xCF\xB8\x03\xFF\xFF\x1E\xCF\xDC\x85\xB9\x44\xDD\xBE\xFF\xFF\xDF\xF7\x7B\x65\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\xBB\x7D\xFF\xFF\xFF\xFF\xFF\xFF\x9B\x75\xBD\xB6\xDC\x85\xBC\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFA\x4C\x3E\xD7\xBD\xB6\xB9\x3C\xFF\xFF\xDF\xF7\xBD\xBE\x59\x2C\xFF\xFF\xBB\x7D\x1C\x96\xFF\xFF\xFF\xFF\x99\x3C\x7B\x6D\xFF\xFF\x1C\x96\x9B\x75\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xD8\x0B\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x7D\xA6\x3A\x5D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x55\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x1E\xCF\xFA\x54\x9F\xE7\xFF\xFF\xFF\xFF\x9D\xAE\x99\x3C\xFF\xFF\xFF\xFF\x7F\xDF\xF8\x13\xFF\xFF\xFF\xFF\xDF\xF7\x59\x2C\xDF\xF7\xFF\xFF\x7F\xE7\x79\x34\x9D\xB6\xFF\xFF\x3A\x5D\xDC\x85\xFF\xFF\xFF\xFF\xFF\xFF\x1A\x55\xDB\x7D\xBF\xF7\x1A\x5D\x5F\xDF\xFF\xFF\x9B\x75\x59\x2C\xDA\x44\xFF\xFF\x1A\x55\x7D\xAE\xFF\xFF\xF8\x13\x5F\xDF\xFF\xFF\xFF\xFF\x5E\xD7\xB7\x0B\xDF\xF7\x1A\x55\x9F\xEF\xFF\xFF\xFF\xFF\x7D\xA6\x1A\x55\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x96\x79\x2C\xF8\x13\xD8\x13\xB8\x03\x97\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x77\x03\x57\x03\x18\x1C\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\x57\x03\xDF\xFF\xFF\xFF\x9F\xEF\x59\x2C\x1C\x96\xFF\xFF\xFA\x4C\x9D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xF8\x1B\xBD\xB6\xFF\xFF\x9B\x6D\xD8\x13\xFF\xFF\xFF\xFF\xDF\xF7\x5B\x65\x7B\x6D\x9D\xAE\x7D\xA6\x7D\xA6\x1E\xCF\x9F\xEF\xDA\x44\x5B\x65\x5E\xD7\x3C\x96\xDA\x44\xFF\xFF\xBB\x7D\xBC\x7D\xFF\xFF\xFF\xFF\xDF\xF7\xFA\x4C\x5E\xDF\x79\x34\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\x38\x24\x1E\xCF\xBB\x7D\xFC\x8D\xFF\xFF\xFF\xFF\xDF\xF7\x57\x03\x7F\xE7\x5D\xA6\xBA\x44\x7D\xA6\x7D\xA6\x5D\xA6\xBD\xB6\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xB8\x0B\x3C\x9E\x7D\xA6\x7D\xA6\x5D\x9E\x9F\xEF\x9D\xAE\x79\x34\x9D\xAE\x7D\xA6\x5C\x9E\xFF\xFF\xFA\x54\x9B\x75\x9D\xAE\x7D\xA6\x5D\x9E\x3E\xD7\xFF\xFF\x9B\x75\xB9\x3C\xFE\xC6\x5D\x9E\xD8\x13\xDD\xBE\xFF\xFF\xFF\xFF\x7F\xDF\x18\x1C\xFF\xFF\xFF\xFF\xDF\xF7\x59\x2C\xBF\xEF\xFF\xFF\xFF\xFF\x3C\x9E\x99\x3C\xBF\xEF\xFF\xFF\x18\x1C\x1C\x96\x1E\xCF\x7B\x6D\x18\x1C\xFF\xFF\x9F\xE7\x3A\x5D\x5F\xDF\xFF\xFF\xFF\xFF\xB7\x03\xDA\x4C\xFF\xFF\x1A\x55\x7D\xA6\xFF\xFF\x9D\xAE\x18\x1C\x9D\xB6\x9D\xB6\xF8\x13\x7D\xA6\xFF\xFF\xBB\x7D\xBA\x44\xBD\xBE\x3E\xCF\xF8\x13\x5E\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\xFE\xC6\x3C\x9E\x9B\x75\xFA\x54\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\x1A\x55\xFA\x54\x7B\x6D\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBF\xEF\xDC\x7D\xFF\xFF\xFF\xFF\xFF\xFF\x5D\xA6\x7F\xE7\xFF\xFF\x1C\x8E\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xDD\xBE\xBD\xB6\xFF\xFF\xFE\xC6\x9D\xAE\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xB6\x9B\x75\x9B\x75\x9B\x75\x9B\x75\x7D\xAE\xFF\xFF\x7F\xE7\xFC\x8D\xBA\x44\xDC\x85\xDF\xF7\xFF\xFF\xFE\xC6\xFE\xC6\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\x9F\xEF\xDC\x7D\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xBD\xBE\xDE\xBE\xFE\xC6\x1E\xCF\xFF\xFF\xFF\xFF\xFF\xFF\x9D\xAE\xBD\xB6\x5F\xDF\x9B\x75\x9B\x75\x9B\x75\x9B\x75\xFC\x8D\xDF\xF7\xFF\xFF\xFF\xFF\xFF\xFF\xBB\x7D\x9B\x75\x9B\x75\x9B\x75\x7B\x6D\x5F\xDF\x7F\xDF\x9B\x75\x9B\x75\x9B\x75\x5B\x65\xFF\xFF\x7D\xA6\x9B\x75\x9B\x75\x9B\x75\x7B\x6D\xBD\xB6\xFF\xFF\xDF\xFF\xDC\x85\x99\x3C\x3A\x5D\x1E\xCF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\xBF\xF7\x1C\x96\xFF\xFF\xFF\xFF\xFF\xFF\x3C\x9E\xDF\xFF\xFF\xFF\xFF\xFF\xDF\xF7\x3C\x96\x7F\xE7\xFF\xFF\xDF\xFF\xBB\x7D\xB9\x3C\x3A\x65\xBF\xEF\xFF\xFF\xDF\xF7\x9D\xB6\xBF\xEF\xFF\xFF\xFF\xFF\x1E\xCF\x5D\xA6\xFF\xFF\x7D\xA6\x1E\xCF\xFF\xFF\xFF\xFF\x3C\x9E\x99\x3C\xFA\x54\x9D\xAE\xFF\xFF\xFF\xFF\xDF\xFF\x7D\xA6\xFA\x54\x3A\x5D\x7F\xDF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
            b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF'
        )

        waveshare_logo = lv.image_dsc_t(
            dict(
                header=dict(cf=lv.COLOR_FORMAT.RGB565, w=200, h=57),
                data_size=22800,
                data=waveshare_logo_data
            )
        )
               
        img = lv.image(self.tile1)
        img.set_src(waveshare_logo) 
        img.align(lv.ALIGN.CENTER, 0, 0)

        lv.screen_load(self.scr)

        # Tile2: Show IMU data
        self.tile2 = self.tv.add_tile(0, 1, lv.DIR.TOP | lv.DIR.BOTTOM)
        table = lv.table(self.tile2)
        table.set_size(85, 265)
        table.align(lv.ALIGN.LEFT_MID, 15 ,0)
        table.remove_flag(lv.obj.FLAG.SCROLLABLE | lv.obj.FLAG.CLICKABLE)
        table.set_cell_value(0, 0, "ACC_X")
        table.set_cell_value(1, 0, "ACC_Y")
        table.set_cell_value(2, 0, "ACC_Z")
        table.set_cell_value(3, 0, "CYRO_X")
        table.set_cell_value(4, 0, "CYRO_Y")
        table.set_cell_value(5, 0, "CYRO_Z")
        table.set_style_bg_color(lv.palette_lighten(lv.PALETTE.BLUE, 1), lv.PART.ITEMS)
        table.add_style(style_imu_table, 0)

        self.table_imu_data = lv.table(self.tile2)
        self.table_imu_data.set_size(78, 265)
        self.table_imu_data.align(lv.ALIGN.CENTER, 15 ,0)
        self.table_imu_data.remove_flag(lv.obj.FLAG.SCROLLABLE | lv.obj.FLAG.CLICKABLE)
        self.table_imu_data.set_cell_value(0, 0, "0")
        self.table_imu_data.set_cell_value(1, 0, "0")
        self.table_imu_data.set_cell_value(2, 0, "0")
        self.table_imu_data.set_cell_value(3, 0, "0")
        self.table_imu_data.set_cell_value(4, 0, "0")
        self.table_imu_data.set_cell_value(5, 0, "0")
        self.table_imu_data.set_style_bg_color(lv.color_make(250, 144, 181), lv.PART.ITEMS)
        self.table_imu_data.add_style(style_imu_table, 0)

        table = lv.table(self.tile2)
        table.set_size(51, 265)
        table.align(lv.ALIGN.RIGHT_MID, -16 ,0)
        table.remove_flag(lv.obj.FLAG.SCROLLABLE | lv.obj.FLAG.CLICKABLE)
        table.set_cell_value(0, 0, "mg")
        table.set_cell_value(1, 0, "mg")
        table.set_cell_value(2, 0, "mg")
        table.set_cell_value(3, 0, "dps") 
        table.set_cell_value(4, 0, "dps")
        table.set_cell_value(5, 0, "dps")
        table.set_style_bg_color(lv.palette_lighten(lv.PALETTE.BLUE, 1), lv.PART.ITEMS)
        table.add_style(style_imu_table, 0)
        
        # Tile3: Some widgets
        self.tile3 = self.tv.add_tile(0, 2, lv.DIR.TOP)
        tabview = lv.tabview(self.tile3);
        tab1 = tabview.add_tab("BAT");
        tab2 = tabview.add_tab("LCD BL");
        
        self.label = lv.label(tab1)
        self.label.set_width(100)
        self.label.set_text("3.33 V")
        self.label.align(lv.ALIGN.CENTER, 20, -10)
        self.label.add_style(style_lable_table, 0)
        
        self.roller = lv.roller(tab2)
        opt = ["1", "2", "3", "4", "5", "6", "7", "8", "9", "10"]
        self.roller.set_options("\n".join(opt), lv.roller.MODE.NORMAL)
        self.roller.set_visible_row_count(3)
        self.roller.align(lv.ALIGN.CENTER, 0, 0)
        self.roller.set_selected(5, lv.ANIM.OFF)
        self.roller.add_style(style_roller, 0)
        self.roller.add_event_cb(self.roller_event_cb, lv.EVENT.VALUE_CHANGED, None)

        # data update
        self.imu_data_update()
        
    # Check update
    def update_check(self,tilex):
        ret = True

        active_tile = self.tv.get_tile_active() 
        if active_tile != tilex:
            ret = False

        return ret

    # Data update
    def imu_data_update(self):
        # Read QMI8658
        xyz = self.IMU.Read_XYZ()  
        
        for i in range(6):
            table_text = "{:4.1f}".format(xyz[i])
            self.table_imu_data.set_cell_value(i, 0, table_text)
            
    def bat_data_update(self):
        # Read ADC
        Vbat= machine.ADC(machine.Pin(28))
        voltage  = Vbat.read_u16()*3.3/65535 * 3
        self.label.set_text(f"{voltage:.2f} V")
                
    # Timer callback
    def imu_data_update_timer_cb(self,t):
        # Check update
        if self.update_check(self.tile2) == True:
            # Update data
            self.imu_data_update()
        
        return
    
    # Timer callback
    def bat_data_update_timer_cb(self,t):
        if self.update_check(self.tile3) == True:
            # Update data
            self.bat_data_update()
            
        return
     
    # Event callback
    def roller_event_cb(self,event):
        value = self.roller.get_selected() + 1
        self.LCD.set_bl_pwm(65535 * value * 10 // 100)





